'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _eventemitter = require('eventemitter3');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _nodeUuid = require('node-uuid');

var _nodeUuid2 = _interopRequireDefault(_nodeUuid);

var _CommandTypes = require('./CommandTypes');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var EMIT = 'ACTIONIZER.EMITTER.NOTIFY';

exports.default = function (initialState) {
  var notify = arguments.length <= 1 || arguments[1] === undefined ? function (emit) {
    emit();
  } : arguments[1];

  var state = initialState;
  var emitter = new _eventemitter2.default();
  var processes = {};

  var subscribe = function subscribe(listener) {
    emitter.on(EMIT, listener);
    return function () {
      emitter.removeListener(EMIT, listener);
    };
  };

  var emit = function emit() {
    emitter.emit(EMIT, state);
  };

  var getState = function getState() {
    return state;
  };

  var setState = function setState(nextState) {
    if (state === nextState) {
      return;
    }
    state = nextState;
    notify(emit);
  };

  var openProcess = function openProcess(action) {
    var actionId = _nodeUuid2.default.v4();
    processes[actionId] = action;
    return actionId;
  };

  var closeProcess = function closeProcess(actionId) {
    if (!processes[actionId]) {
      return;
    }
    processes[actionId].return();
    delete processes[actionId];
  };

  var dispatch = function dispatch(action) {
    var actionId = openProcess(action);

    var step = function step(result) {
      var done = result.done;
      var value = result.value;


      if (done) {
        return closeProcess(actionId);
      }

      switch (value.type) {
        case _CommandTypes.SELECT:
          step(action.next(value.selector(state)));
          break;
        case _CommandTypes.PUT:
          setState(value.nextState);
          step(action.next(state));
          break;
        case _CommandTypes.CALL:
          value.fn.apply(value, _toConsumableArray(value.args)).then(function (val) {
            step(action.next(val));
          }).catch(function (e) {
            step(action.throw(e));
          });
          break;
        case _CommandTypes.FORK:
          step(action.next(dispatch(value.actionCreator.apply(value, _toConsumableArray(value.args)))));
          break;
        case _CommandTypes.CANCEL:
          closeProcess(value.actionId);
          step(action.next());
          break;
        default:
          step(action.throw(new Error('Received unknown command.')));
          break;
      }
    };

    step(action.next());
    return actionId;
  };

  return {
    subscribe: subscribe,
    getState: getState,
    dispatch: dispatch
  };
};